;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;                                                                                    ;
; ***Sledge Brother***                                                               ;
;  Author of Sprite:	Schwa                                                        ;
;   Original Sprite:	mikeyk                                                       ;
;                                                                                    ;
; What this does:                                                                    ;
; He throws hammers at Mario, occasionally jumping into the air.                     ;
;                                                                                    ;
; This sprite will shake the ground whenever he falls and lands on it, stunning      ;
; Mario if he's on the ground. He also takes two stomps to defeat-- the first stomp  ;
; will knock him backwards, the second will defeat him.                              ;
;                                                                                    ;
; Extra Bit Clear:                                                                   ;
; He throws 1 hammer at a time like a normal Hammer Bro.                             ;
;                                                                                    ;
; Extra Bit Set:                                                                     ;
; He throws hammers 3 at a time. The throw formation is of a triangle shape, and     ;
; they cover the inside of his arc unlike before, so the only reliable way to dodge  ;
; is by weaving in-between the hammers.                                              ;
;                                                                                    ;
; Extra Property Byte:                                                               ;
; This one sets the HP the sledge brother has got.                                   ;
; Keep in mind that each stomp takes 3 HP so if you want to make the sledge survive  ;
; three hits, you have to set it to nine instead of three.                           ;
;                                                                                    ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;
; variables and Ram ;
;;;;;;;;;;;;;;;;;;;;;

	!r_JUMP		= !163E
	!r_THROW		= !1504
	!r_HP		= !1528
	!r_FLASH		= !1534
	!r_BIT		= !7FAB10

	!v_STUN		= $38
	!v_FLASHTIME	= $6F
	!v_HIT_YSPD	= $D8
	!v_HIT_SFX	= $20
	!v_SHOW_HMR	= $20      ;time to display the hammer before it is thrown
	!v_HAMMER_TILE	= $6D
	!v_HAMMER_PROP	= $07

t_XSPEED:
	db $00,$FC,$00,$04     ;rest at bottom, moving up, rest at top, moving down
t_XSPEED_KILLED:
	db $F0,$10	; when killed with star
t_TIME_IN_POS:
	db $48,$10,$48,$10     ;moving up, rest at top, moving down, rest at bottom
t_UNTIL_THROW:
	db $43,$3C
t_BOUNCE:
	db $E4,$1C	;x speed to give mario when he stomps the sprite
t_XOFFSET_LO:
	db $F4,$0C
t_XSPEED_HAMMER:
	db $12,$EE,$10,$F0,$14,$EC,$1C,$E4
t_YSPEED_HAMMER:
	db $D0,$D0,$D2,$D2,$C0,$C0,$C8,$C8
t_TILEMAP:
	db $A6,$A9,$E0,$E1,$A6,$A9,$E3,$E4,$A6,$A7,$E0,$E1,$A6,$A7,$E3,$E4
	db $A6,$A9,$E0,$E1,$A6,$A9,$E3,$E4,$A6,$A7,$E0,$E1,$A6,$A7,$E3,$E4
t_XDISP:
	db $FC,$04,$FC,$04,$FC,$04,$FC,$04,$FC,$04,$FC,$04,$FC,$04,$FC,$04
	db $04,$FC,$04,$FC,$04,$FC,$04,$FC,$04,$FC,$04,$FC,$04,$FC,$04,$FC
t_YDISP:
	db $F0,$F0,$00,$00,$F0,$F0,$00,$00,$F0,$F0,$00,$00,$F0,$F0,$00,$00
	db $F0,$F0,$00,$00,$F0,$F0,$00,$00,$F0,$F0,$00,$00,$F0,$F0,$00,$00
t_PROPFLASH:
	db $00,$02,$04,$06,$08,$0A,$0C,$0E
t_PROPERTIES:
	db $40,$00             ;xyppccct format

;;;;;;;;;;;;;;;;
; init routine ;
;;;;;;;;;;;;;;;;

	print "INIT ",pc
	%SubHorzPos()
	TYA
	STA !157C,x

	LDA !1588,x
	ORA #$04
	STA !1588,x

	STZ !1602,x

	TXA
	AND #$03
	;ASL A
	;ASL A
	ASL A
	ASL A
	PHA
	ASL A
	ADC #$32
	STA !r_THROW,x

	PLA
	ADC #$FE
	STA !r_JUMP,x

	RTL

;;;;;;;;;;;;;;;;
; main routine ;
;;;;;;;;;;;;;;;;

	print "MAIN ",pc
HAMMER_BRO_JSL:
	PHB			; \
	PHK			;  | main sprite function, just calls local subroutine
	PLB			;  |
	JSR DecrementTimers	;  |
	JSR START_HB_CODE	;  |
	PLB			;  |
	RTL			; /

DecrementTimers:
	LDA !14C8,x
	CMP #$08
	BNE DoneDec2
	LDA $9D
	BNE DoneDec2

	LDA !r_FLASH,x
	BEQ DoneDec1
	DEC !r_FLASH,x
DoneDec1:
	LDA !r_THROW,x
	BEQ DoneDec2
	DEC !r_THROW,x
DoneDec2:
	RTS
	
;;;;;;;;;;;;;;;;;;;;
; main sprite code ;
;;;;;;;;;;;;;;;;;;;;

STAR:
	INC !r_THROW,x
RETURN1:
	RTS                    
START_HB_CODE:
	JSR SUB_GFX	; draw hammer bro gfx
	LDA !14C8,x	; \ if hammer bro status != 8...
	CMP #$02	;  |   ... not (killed with spin jump [4] or star[2])
	BEQ STAR	;  |
	CMP #$08	;  |
	BNE RETURN1	; /    ... return
	LDA $9D		; \ if sprites locked...
	BNE RETURN1	; /    ... return

	%SubHorzPos()	; \ always face mario
	TYA			;  | 
	STA !157C,x		; /

	LDA #$03
	%SubOffScreen()		; only process hammer bro while on screen
	INC !1570,x             ; increment number of frames hammer bro has been on screen
                    
	LDA !1570,x             ; \ calculate which frame to show:
	LSR A                   ;  | 
	LSR A                   ;  | 
	LSR A                   ;  | 
	LSR A                   ;  | 
	AND #$01                ;  | update every 16 cycles if normal
	STA !1602,x             ; / write frame to show

	LDA !r_THROW,x		; \ if time until spit >= $10
	CMP #!v_SHOW_HMR		;  |   just go to normal walking code
	BCS JUMP_SPRITE		; /
	INC !1602,x 
	INC !1602,x 
                
	LDA !r_THROW,x		; \ throw hammer if it's time
	BNE NO_RESET		;  |
	LDY !C2,x		;  |
	LDA t_UNTIL_THROW,y	;  |
	STA !r_THROW,x		;  |
NO_RESET:
	CMP #$01		;  |
	BNE JUMP_SPRITE		;  |
	LDA !C2,x		;  |
	EOR #$01		;  |
	STA !C2,x		;  |
	JSR SUB_WEAPON		; /

JUMP_SPRITE:
	LDA !r_JUMP,x
	CMP #$40                ; \ just go to normal walking code
	BCS WALK_SPRITE         ; /
	LDA !1602,x             ; \ lock image while jumping
	ORA #$01                ;  |
	STA !1602,x             ; /
	INC !1540,x             ; we didn't move the sprite this frame, so we don't want a decrement
	STZ !B6,x               ; stop sprite from moving
	LDA !r_JUMP,x
	CMP #$38
	BNE NO_JUMP2
	LDA #$B0                ; \  y speed
	STA !AA,x               ; /
	BRA APPLY_SPEED
NO_JUMP2:
	CMP #$00
	BNE APPLY_SPEED
	LDA #$FE
	STA !r_JUMP,x

	BRA APPLY_SPEED

WALK_SPRITE:
	LDA !r_FLASH,x	  ; \ completely skip x speed code if flashing
	BNE FLASH_SPEED	  ; / to allow sprite to get knocked backwards

	LDA !151C,x             ;
	TAY                     ;
	LDA !1540,x             ; 
	BEQ CHANGE_SPEED        ; 
	LDA t_XSPEED,y           ; \ set x speed
	STA !B6,x               ; /
	BRA APPLY_SPEED

CHANGE_SPEED:
	LDA t_TIME_IN_POS,y
	STA !1540,x
	LDA !151C,x
	INC A
	AND #$03
	STA !151C,x

	BRA APPLY_SPEED

FLASH_SPEED:
	LDA !1588,x	; \
	AND #$04	;  | if on the ground AND not moving upward...
	BEQ APPLY_SPEED	;  |
	LDA !AA,x	;  |
	BMI APPLY_SPEED	; /

	STZ !B6,x	; ...stop sprite from being knocked back
                    
APPLY_SPEED:
	LDA !1588,x             ; push ground status so we can compare later
	PHA                     ;

	JSL $01802A|!BankB             ; update position based on speed values

	LDA !1588,x             ; if on the ground now...
	AND #04                 ;
	BEQ IN_AIR              ;
	PLA                     ; ...and not on the ground before
	AND #04                 ;
	BNE CHECK_TOUCH         ;
	LDA #!v_STUN       ; \ shake ground
	STA $1887|!Base2               ; /
                    
	LDA #$09                ; \ play sound effect
	STA $1DFC|!Base2               ; /
                    
	LDA $77
	AND #$04
	BEQ CHECK_TOUCH
	LDA #!v_STUN       ; set timer to freeze mario
	STA $18BD|!Base2

	BRA CHECK_TOUCH
IN_AIR:
	PLA

CHECK_TOUCH:
	LDA !1588,x             ; \ if hammer bro is touching the side of an object...
	AND #$03                ;  |
	BEQ DONT_CHANGE_DIR     ;  |
	LDA !151C,x		;  |
	INC A			;  |
	AND #$03		;  |
	STA !151C,x		; /
                    
DONT_CHANGE_DIR:
	JSL $018032|!BankB	; interact with other sprites               

	LDA !r_FLASH,x	; \ no Mario contact allowed if flashing
	BNE NO_CONTACT	; / 

	JSL $01A7DC|!BankB	; \ check for mario/hammer bro contact
	BCC NO_CONTACT	; / do nothing if no contact

	BRA CONTACT
NO_CONTACT:
	RTS	; return

CONTACT:
        %SubVertPos()	; \
        LDA $0E			;  | if mario isn't above sprite, and there's vertical contact...
        CMP #$E6		;  |     ... sprite wins
        BPL SPRITE_WINS		; /
        LDA $7D			; \ if mario speed is upward, return
        BMI RETURN2		; /

	LDA #!v_FLASHTIME	; \ prevent the Double-Edge glitch from happening
	STA !154C,x		; /
	JSR SUB_STOMP_PTS       ; give mario points

	LDA $15		; Save controller state
        PHA
        ORA #$C0	; Set holding X/Y and A/B
        STA $15
        JSL $01AA33|!BankB	; set mario speed
	PLA
	STA $15

	%SubHorzPos()
	TYA
	EOR #$01
	TAY
        LDA t_BOUNCE,y	;  | bounce away horizontally
        STA $7B			; /

	JSL $01AB99|!BankB             ; display contact graphic

        LDA !r_HP,x	; \  
	CMP !7FAB28,x	;  | If this is the second stomp (i.e. has taken 3 or more dmg)...
	BCC STOMP1	; /  

        LDA #$02	; \  
        STA !14C8,x	;  | ...kill the sprite and finish
        STZ !B6,x	;  | 
	STA !AA,x	;  | 
RETURN2:
	RTS		; /  

SPRITE_WINS:
	LDA !154C,x	; \ if disable interaction set...
        ORA !15D0,x	;  |   ...or sprite being eaten...
        BNE RETURN2	; /   ...return

        LDA $1490|!Base2	; \ Branch if Mario has a star
        BNE HAS_STAR	; /

        JSL $00F5B7|!BankB	; damage Mario

	RTS

STOMP1:
	LDA !r_HP,x	; \
	CLC		;  | deal 3 "damage" (applies to both stomping and fireballs)
	ADC #$03	;  |
	STA !r_HP,x	; /

	LDA #!v_HIT_SFX	; \ play sound effect
	STA $1DF9|!Base2	; /

	LDA #!v_HIT_YSPD	; \ set y speed
	STA !AA,x	; /

	%SubHorzPos()  	; \
	LDA t_XSPEED_KILLED,y 	;  | set x speed
	STA !B6,x		; /

	LDA #$FE	; \ reset jump timer
	STA !r_JUMP,x	; / 
	LDA #$62	; \ delay next hammer throw
	STA !r_THROW,x	; / 

	LDA #!v_FLASHTIME  ; \ cause sprite to flash palettes
	STA !r_FLASH,x	  ; /

	RTS

HAS_STAR:
	LDA #$02	; \ status = 2 (being killed by star)
	STA !14C8,x	; /
	LDA #$D0	; \ set y speed
	STA !AA,x	; /
	%SubHorzPos()	; get new direction
	LDA t_XSPEED_KILLED,y	; \ set x speed based on direction
	STA !B6,x		; /
	INC $18D2|!Base2	; increment number consecutive enemies killed
	LDA $18D2|!Base2	; \
	CMP #$08	;  | if consecutive enemies stomped >= 8, reset to 8
	BCC NO_RESET2	;  |
	LDA #$08	;  |
	STA $18D2|!Base2	; /   
NO_RESET2:
	JSL $02ACE5	  ; give mario points
	LDY $18D2|!Base2	  ; \ 
	CPY #$08	  ;  | if consecutive enemies stomped < 8 ...
	BCS NO_SOUND2	  ;  |
	LDA STAR_SOUNDS,y ;  |    ... play sound effect
	STA $1DF9|!Base2	  ; /
NO_SOUND2:
	RTS	; final return

;;;;;;;;;;;;;;;;;;;;;
; hammer routine    ;
; (from Hammer Bro) ;
;;;;;;;;;;;;;;;;;;;;;

SUB_WEAPON:
	LDA !157C,x	; \ $04 controls the value loaded from the Speed Table
	STA $04		; /

	LDA #$01	; \ $05 = how many times to loop
	STA $05		; /

	LDA !157C,x ; \ $06 is the direction the sprite faces
	STA $06     ; /

	LDA !r_BIT,x	; \
	AND #$04	;  | loop if extra bit set
	BEQ NO_LOOP	; /

	INC $05	; \ increase loop counter
	INC $05	; /
	INC $04	; \ use different Speed Table settings
	INC $04	; /
NO_LOOP:
	LDY #$E8
	LDA !157C,x
	BNE LABEL9
	LDY #$18
LABEL9:
	STY $00

LABEL8:
	LDX $06
	LDA t_XOFFSET_LO,x	;  | set x position
	STA $00	; /

	LDA #$E7	; \ set y position
	STA $01		; /

	LDX $04			; \ set x speed
	LDA t_XSPEED_HAMMER,x	;  |
	STA $02		; /

	LDA t_YSPEED_HAMMER,x	; \ set y speed
	STA $03		; /

	LDX $15E9|!Base2
	LDA #$04
	%SpawnExtended()
	BRA LABEL10
LABEL6:
	RTS
LABEL10:
	DEC $05		; \
	LDA $05		;  | decrement loop counter
	BEQ LABEL6	; /

	INC $04		; \
	INC $04		;  | if still looping, change Speed Table values
	BRA LABEL8	; /

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; graphics routine
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SUB_GFX:
	%GetDrawInfo()	; after: Y = index to sprite tile map ($300)
				;      $00 = sprite x position relative to screen boarder 
				;      $01 = sprite y position relative to screen boarder  
	LDA !1602,x             ; \
	ASL A                   ;  | $03 = index to frame start (frame to show * 2 tile per frame)
	ASL A
	STA $03                 ; /
	LDA !157C,x             ; \ $02 = sprite direction
	STA $02                 ; /
	BNE NO_ADD
	LDA $03
	CLC
	ADC #$10
	STA $03
NO_ADD:
	PHX                     ; push sprite index

	LDX #$03                ; loop counter = (number of tiles per frame) - 1
LOOP_START:
	PHX                     ; push current tile number
	TXA                     ; \ X = index to horizontal displacement
	ORA $03                 ; / get index of tile (index to first tile of frame + current tile number)
FACING_LEFT:
	TAX                     ; \ 

	LDA $00                 ; \ tile x position = sprite x location ($00)
	CLC                     ;  |
	ADC t_XDISP,x         ;  |
	STA $0300|!Base2,y             ; /

	LDA $01                 ; \ tile y position = sprite y location ($01) + tile displacement
	CLC                     ;  |
	ADC t_YDISP,x         ;  |
	STA $0301|!Base2,y             ; /
                    
	LDA t_TILEMAP,x           ; \ store tile
	STA $0302|!Base2,y             ; / 

	LDX $02                 ; \
	LDA t_PROPERTIES,x        ;  | get tile t_PROPERTIES using sprite direction
	LDX $15E9|!Base2               ;  |
	ORA !15F6,x             ;  | get palette info
	ORA $64                 ;  | put in level t_PROPERTIES
	STA $0303|!Base2,y             ; / store tile t_PROPERTIES

	LDA !r_FLASH,x	; \ check to see if the sprite should be flashing
	BEQ NO_FLASH	; /

	LDA $14		  ; \
	AND #$07	  ;  | cycle through all 8 sprite palettes
	TAX		  ;  |
	LDA $0303|!Base2,y	  ;  |
	AND #$F1	  ;  | <-- F1 = 11110001 = clear all Palette-related bits
	ORA t_PROPFLASH,x  ;  |
	STA $0303|!Base2,y	  ; /

NO_FLASH:
	PLX                     ; \ pull, X = current tile of the frame we're drawing
	INY                     ;  | increase index to sprite tile map ($300)...
	INY                     ;  |    ...we wrote 1 16x16 tile...
	INY                     ;  |    ...sprite OAM is 8x8...
	INY                     ;  |    ...so increment 4 times
	DEX                     ;  | go to next tile of frame and loop
	BPL LOOP_START          ; / 

	PLX                     ; pull, X = sprite index
                    
	LDA !r_THROW,x
	CMP #$02
	BCC NO_SHOW_HAMMER
	CMP #30
	BCS NO_SHOW_HAMMER
	LDA !1602,x
	CMP #$02
	BCS SHOW_HAMMER_TOO

NO_SHOW_HAMMER:
	LDY #$02                ; \ 02, because we didn't write to 460 yet
	LDA #$03                ;  | A = number of tiles drawn - 1
	;JSL $01B7B3             ; / don't draw if offscreen
	;RTS                     ; return
	BRA Draw                ; Actually, just jump to the existing routine instead

HAMMER_OFFSET:
	db $F4,$0C

SHOW_HAMMER_TOO:
	PHX

	LDA $00
	LDX $02
	CLC
	ADC HAMMER_OFFSET,x
	STA $0300|!Base2,y
                    
	LDA $01                 ; \ tile y position = sprite y location ($01) + tile displacement
	CLC                     ;  |
	ADC #$E8
	STA $0301|!Base2,y             ; /
                    
	LDA #!v_HAMMER_TILE                ; \ store tile
	STA $0302|!Base2,y             ; / 

	LDA #!v_HAMMER_PROP
	CPX #$00
	BNE NO_FLIP_HAMMER     
	ORA #$40
NO_FLIP_HAMMER:
	ORA $64                 ;  | put in level t_PROPERTIES
	STA $0303|!Base2,y             ; / store tile t_PROPERTIES
                    
	PLX
	;INY                     ;  | increase index to sprite tile map ($300)...
	;INY                     ;  |    ...we wrote 1 16x16 tile...
	;INY                     ;  |    ...sprite OAM is 8x8...
	;INY                     ;  |    ...so increment 4 times

	LDY #$02                ; \ 02, because we didn't write to 460 yet
	LDA #$04                ;  | A = number of tiles drawn - 1
Draw:
	JSL $01B7B3|!BankB             ; / don't draw if offscreen
	RTS                     ; return

;;;;;;;;;;;;;;;;;;
; points routine ;
;;;;;;;;;;;;;;;;;;

STAR_SOUNDS:
	db $00,$13,$14,$15,$16,$17,$18,$19

SUB_STOMP_PTS:
	PHY                      
        LDA $1697|!Base2               ; \
        CLC                     ;  | 
        ADC !1626,x             ; / some enemies give higher pts/1ups quicker??
        INC $1697|!Base2               ; increase consecutive enemies stomped
        TAY                     ;
        INY                     ;
        CPY #$08                ; \ if consecutive enemies stomped >= 8 ...
        BCS NO_SOUND            ; /    ... don't play sound 
        LDA STAR_SOUNDS,y       ; \ play sound effect
        STA $1DF9|!Base2               ; /   
NO_SOUND:
	TYA                     ; \
        CMP #$08                ;  | if consecutive enemies stomped >= 8, reset to 8
        BCC NO_RESET3           ;  |
        LDA #$08                ; /
NO_RESET3:
	JSL $02ACE5|!BankB	; give Mario points
        PLY                     
        RTS                     ; return

